# THIS FILE IS GENERATED FROM TRAITS SETUP.PY
version = '5.1.0'
full_version = '5.1.0'
git_revision = 'bc896728d443c8417f175c799353b2e55b63833f'
is_released = True

if not is_released:
    version = full_version
